Attribute VB_Name = "modSchedule"
Option Explicit
'Main Form Width
Public Const FORM_WIDTH = 440

'Control sizing constants
Public Const COLUMN_GUTTER = 1
Public Const COLUMN_MEMO = 48
Public Const FRAME_RATIO = 0.8
Public Const LIST_RATIO_H = 0.95
Public Const LIST_RATIO_V = 0.9
Public Const GUTTER_HORZ = 6
Public Const GUTTER_VERT = 6

'Move constants for frmMain.PollForDataChanges
Public Const MOVE_NEXT = 1
Public Const MOVE_PREVIOUS = 0

'View increment constants
Public Const QUARTER_HOUR = 15
Public Const HALF_HOUR = 30
Public Const FULL_HOUR = 60

'Database constants
Public Const DATABASE_FILENAME = "vbSched.mdb"
Public Const DQ = """"

'Error constants
Public Const APPOINTMENT_MADE = 0
Public Const ERR_MATCHING_TIME = 1
Public Const ERR_NULL_APPOINTMENT = 2
Public Const ERR_APPOINTMENT_CONFLICT = 3

'Sizing globals
Public gAppointmentDelta As Integer ' Time change granularity
Public gColOneSize As Integer 'Global for size of 1st column
Public gColTwoSize As Integer 'Global for size of 2nd column
Public gColThreeSize As Integer 'Global for size of 3rd column
Public gAppointmentField As Integer 'Global appointment width
Public gComboBox As ComboBox 'The combo box in edit mode
Public gComboText As String 'Global value for gbComboEditOn examination
Public gLeftFramesWidth As Integer 'Global left frame's width
Public gGutterHorz As Integer
Public gGutterVert As Integer

'Appointment String Globals
Public gOpenApptLine As String ' String denoting an open appt
Public gClosedApptLine As String ' String denoting a closed appt

'Semaphores
Public gbAddOn As Boolean 'Indicates a contact ADD is in progress
Public gbEditOn As Boolean 'Indicates an app wide EDIT in progress
Public gbKBEditOn As Boolean 'Indicates that a contact is being edited
Public gbComboEditOn As Boolean 'Indicate a combo dropdown

Public gbInitOver As Boolean 'Indicates that all the controls are loaded
                             'and ready for poplulating

Public gCurrentApptIndex As Integer 'Index of item being edited
Public gcrContactBuffer As ContactRec 'Holds last old record data

Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long

Type POINTAPI
        X As Long
        Y As Long
End Type

Type ContactRec
    fn As String   'First Name
    ln As String   'Last Name
    cmp As String  'Company
    addr As String 'Address
    city As String 'City
    st As String   'State
    zip As String  'Zip
    ph As String   'Phone
    fax As String  'Fax
    em As String   'Email
End Type

Type Appt
    num As Integer
    sTime As Double
    eTime As Double
    Comment As String
End Type
'************************************************
'Sub/Function Name: SetContactBuffer
'
'Arguments: cr as ContactRec    A user defined type containing
'                               contact data.
'
'Return: None
'
'Remarks:   SetContactBuffer is used by VBScheduler to keep an
'           original copy of contact record data that is being
'           edited in a global instance of a user defined type
'           ContactRec, gcrContactBuffer. If the user wants to
'           revert to pre-edited data, the program calls
'           GetContactBuffer.
'
'VB concept demonstrated: User defined type
'                         Variables with global scope
'
'Progammer/Creator: Bob Reselman
'Copyright (c)1997 Macmillian Publishing
'************************************************
Public Sub SetContactBuffer(cr As ContactRec)
    'Set the global user defined type, gcrContactBuffer with
    'the data passed by cr.
    gcrContactBuffer.fn = cr.fn
    gcrContactBuffer.ln = cr.ln
    gcrContactBuffer.cmp = cr.cmp
    gcrContactBuffer.addr = cr.addr
    gcrContactBuffer.city = cr.city
    gcrContactBuffer.st = cr.st
    gcrContactBuffer.zip = cr.zip
    gcrContactBuffer.ph = cr.ph
    gcrContactBuffer.fax = cr.fax
    gcrContactBuffer.em = cr.em
End Sub

'************************************************
'Sub/Function Name: ParseName
'
'Arguments: strFullString As String     A string containing the value
'                                       to be parsed.
'
'           strFirstName As String      A string to which to pass the
'                                       first value parsed out of
'                                       strFullString.
'
'           strLastName As String       A string to which to pass the
'                                       second value parsed out of
'                                       strFullString.
'
'           strPhone As String          A string to which to pass the
'                                       third value parsed out of
'                                       strFullString.
'
'Return: True if parse is successful
'
'Remarks: ParseName is used convert the value of the string contain-
'         ing the First Name, Last Name and Phone of a contact into
'         three distinct strings. This function is typically used to
'         set the value in the cboName combo box to the First Name and
'         Last Name text values during edit mode. It is also used to
'         create parameters for SQL statements.
'
'VB concept demonstrated:   Len()
'                           Mid()
'                           Trim()
'                           For...Next
'************************************************
Public Function ParseName(strFullString As String, _
                          strFirstName As String, _
                          strLastName As String, _
                          strPhone As String) As Boolean
    Dim CommaPos%
    Dim i%
    Dim j%
    Dim sBuffer(3) As String
    Dim sChar As String
    
    On Error GoTo ParseNameErr:
    'Add a comma on the end
    strFullString = strFullString & ","
    For i = 1 To Len(strFullString)
        'check out each character, if it is not a comma add
        'it to the buffer string
        sChar = Mid(strFullString, i%, 1)
        If sChar <> "," Then
            sBuffer(j%) = sBuffer(j%) & sChar
        Else
            j% = j% + 1
        End If
    Next i%
    
    strLastName = Trim(sBuffer(0))
    strFirstName = Trim(sBuffer(1))
    strPhone = Trim(sBuffer(2))
    ParseName = True
    Exit Function
ParseNameErr:
    MsgBox "Parsing Error", vbCritical
End Function

'************************************************
'Sub/Function Name: MoveContactRecord
'
'Arguments: f as Form               A form containing the contact
'                                   combo box
'
'           c as ComboBox           A combo box containing a contact
'
'           Direction as Integer    0 = previous
'                                   1 = next
'
'Return: None
'
'Remarks:   MoveContactRecord is used to navigate backward and forward
'           through the contact table. The sub moves through the list
'           of combo box, c doing an SQL query on each value in the
'           list as it tranverses. Each query field retrieved is
'           assigned to its associated text box.
'
'VB concept demonstrated: If...Then...Else
'                         Select Case
'                         Exit Sub
'                         MsgBox
'                         vbCrLf
'                         String concatentation
'                         User defined error messages
'                         ComboBox.List
'                         ComboBox.ListIndex
'************************************************
Public Sub MoveContactRecord(f As Form, _
                             c As ComboBox, _
                             Direction As Integer)
    Dim b As Boolean
    Dim FullName As String
    Dim FirstName As String
    Dim LastName As String
    Dim Phone As String
    Dim Msg As String
    
    'Check Direction
    Select Case Direction
        Case 0
            'If it is 0, decrement
            If c.ListIndex <> 0 Then
                c.ListIndex = c.ListIndex - 1
            End If
        Case 1
            'If it is 1, increment
            If c.ListIndex <> c.ListCount - 1 Then
                c.ListIndex = c.ListIndex + 1
            End If
        Case Else
            Msg = "Function: MoveContactRecord"
            Msg = Msg & vbCrLf & vbCrLf
            Msg = Msg & "You have entered a erroneous Direction value"
            'Report error
            MsgBox Msg, vbCritical, "Parameter Error"
            'Get out of town
            Exit Sub
    End Select
    'Fill the Text boxes
    FullName = c.List(c.ListIndex)
    b = ParseName(FullName, FirstName, LastName, Phone)
     If b Then
        'Attach the info for that name to the text boxes.
        Call AttachName(LastName, FirstName, Phone, f.dataMain)
    End If

End Sub
'************************************************
'Sub/Function Name: EditRecord
'
'Arguments: r as Recordset      A recordset
'           cr as ContactRec    A user defined type
'
'
'Return: None
'
'Remarks: EditRecord encapsulates the edit
'         method of a recordset.
'
'VB concept demonstrated: Recordset.Edit
'                         Recordset.Update
'                         User defined types
'************************************************
Public Sub EditRecord(r As Recordset, cr As ContactRec)
    'Set the recordset to edit mode
    r.Edit
    'Set the fields
    r.Fields("tblContacts.FirstName") = cr.fn
    r.Fields("tblContacts.LastName") = cr.ln
    r.Fields("Company") = cr.cmp
    r.Fields("Address") = cr.addr
    r.Fields("City") = cr.city
    r.Fields("State") = cr.st
    r.Fields("Zip") = cr.zip
    r.Fields("tblContacts.Phone") = cr.ph
    r.Fields("Fax") = cr.fax
    r.Fields("Email") = cr.em
    'Write the record
    r.Update
End Sub '
'************************************************
'Sub/Function Name:
'
'Arguments: cr as ContactRec    A user defined type holding
'                               contact record data.
'
'Return: The retrieved ContactRec is set to cr.
'
'Remarks:   GetContactBuffer retrieves the values from the global
'           user defined type gcrContactBuffer of type, ContactRec.
'           This sub is typically used to retrieve data for an edit
'           revert. To set data to the global Contact Buffer, use
'           SetContactBuffer.
'
'VB concept demonstrated: User defined types
'                         Variables with global scope
'************************************************
Public Sub GetContactBuffer(cr As ContactRec)
    'Set the passed type, cr to a global ContactRec type,
    'gcrContactBuffer.
    cr.fn = gcrContactBuffer.fn
    cr.ln = gcrContactBuffer.ln
    cr.cmp = gcrContactBuffer.cmp
    cr.addr = gcrContactBuffer.addr
    cr.city = gcrContactBuffer.city
    cr.st = gcrContactBuffer.st
    cr.zip = gcrContactBuffer.zip
    cr.ph = gcrContactBuffer.ph
    cr.fax = gcrContactBuffer.fax
    cr.em = gcrContactBuffer.em
End Sub
'************************************************
'Sub/Function Name: ReInitDBtoApp
'
'Arguments: d As Data   A data control containing the contact
'                       table.
'
'
'Return:  None
'
'Remarks: ReInitDBtoApp reintializes the database to the data
'         control, retrieving all the contacts from the table,
'         tblContacts and sorting them on the last name of the
'         contact.
'
'
'VB concept demonstrated: Using an SQL statement as a record-
'                         source for a Data control
'************************************************'
Public Sub ReInitDBtoApp(d As Data)

    Dim MySQL As String
    'Create a database that retrieves all the contacts from
    'the table, tblContacts.
    MySQL = "SELECT * FROM tblContacts "
    MySQL = MySQL & "ORDER BY tblContacts.LastName"
    'Assign the SQL statement to the recordsource property
    'of the Data control.
    d.RecordSource = MySQL
    'Hook it up
    d.Refresh
End Sub '************************************************
'************************************************'
'Sub/Function Name: PopluateDateCombo
'
'Arguments: iMonth as Integer   An Integer representing month of the
'                               year, NOT in serial time notation
'
'           iYear as Integer    An integer representing a given year.
'                               NOT in serial notation
'
'           c as ComboBox       A combo box to which to set the dates
'                               of a given month
'
'Return: None
'
'Remarks:   PopulateDateCombo populates a combo box with the dates
'           of a given month within a given year. The function
'           accommodates leap year months, 30 day and 31 day months.
'
'VB concept demonstrated:   Select Case
'                           For...Next
'                           If...Then...Else
'                           ComboBox.AddItem
'                           ComboBox.Clear
'************************************************'
Public Sub PopluateDateCombo(iMonth As Integer, _
                             iYear As Integer, _
                             c As ComboBox)

    Dim DaysInMonth% 'Variable to hold days in month
    Dim i%           'counter
    
    'Find out how many days in the month
    Select Case iMonth
        'A 31 Day month
        'Jan, March, May, July, Aug, Oct, Dec
        Case 1, 3, 5, 7, 8, 10, 12
            DaysInMonth% = 31
        'April, June, Sept, November
        Case 4, 6, 9, 11
            DaysInMonth% = 30
        'February
        Case 2
            If IsLeapYear(iYear) Then
                DaysInMonth% = 29
            Else
                DaysInMonth% = 28
            End If
    End Select
    'Empty the combo box
    c.Clear
    'Fill it with the numeric days in the month
    For i% = 1 To DaysInMonth%
        c.AddItem CStr(i%)
    Next i%
End Sub
'************************************************
'Sub/Function Name: AddRecord
'
'Arguments: d as Data           A data control
'           cr as ContactRec    A user defined type
'
'Return: None
'
'Remarks:   AddRecord encapsulates the process of
'           adding a record to a data control's
'           database using SQL.
'
'
'VB concept demonstrated:   SQL statements
'                           Data.Recordsource
'                           Data.Refresh
'                           Data.Recordset.AddNew
'                           Data.Recordset.Fields()
'                           Data.Recordset.Update
'************************************************
Public Sub AddRecord(d As Data, cr As ContactRec)
    Dim MySQL As String  'An variable to hold the SQL string
    'Make a SQL statement the retrieves all the records
    'from the table, tblContacts
    MySQL = "SELECT * FROM tblContacts"
    'Assign the SQL statement to the data control's
    'recordsource property
    d.RecordSource = "tblContacts"
    'Put it into effect
    d.Refresh
    'Make a new record in that table
    d.Recordset.AddNew
    'Set the values for that record
    d.Recordset.Fields("FirstName") = cr.fn
    d.Recordset.Fields("LastName") = cr.ln
    d.Recordset.Fields("Company") = cr.cmp
    d.Recordset.Fields("Address") = cr.addr
    d.Recordset.Fields("City") = cr.city
    d.Recordset.Fields("State") = cr.st
    d.Recordset.Fields("Zip") = cr.zip
    d.Recordset.Fields("Phone") = cr.ph
    d.Recordset.Fields("Fax") = cr.fax
    d.Recordset.Fields("Email") = cr.em
    'Make the record addition permanent
    d.Recordset.Update
End Sub

'************************************************
'Sub/Function Name: IsLeapYear
'
'Arguments: iYear as Integer    An integer representing a year
'                               i.e., 1997
'
'Return: True, if the submitted integer represents a leap year
'
'Remarks: This function is used to determine if a year is a leap
'         year. Be advised that the argument, iYear is not of
'         serial date notation. It is simply a year in integer
'         notation. i.e., 1997, 1986, 1995
'
'VB concept demonstrated: MOD operator
'************************************************'
Public Function IsLeapYear(iYear As Integer) As Boolean
    If iYear Mod 4 = 0 Then
        IsLeapYear = True
    Else
        IsLeapYear = False
    End If
End Function
'************************************************
'Sub/Function Name: InitComboAsDb
'
'Arguments: c as ComboBox   A combo box holding the contact
'                           list.
'
'           d as Data       A data control
'
'Return: None
'
'Remarks:   InitComboAsDB retrieves all the records from the
'           data control and assigns them as items in the combo box,
'           c.
'
'
'VB concept demonstrated:   ComboBox.AddItem
'                           ComboBox.Clear
'                           ComboBox.ListIndex
'                           Data.Recordset.EOF
'                           Data.Recordset.Fields()
'                           Data.Recordset.MoveNext
'                           While...Wend
'                           Not
'************************************************'
Public Sub InitComboAsDb(c As ComboBox, d As Data)
    Dim NumOfRec%
    Dim i%
    Dim First As String
    Dim Last As String
    Dim Phone As String
    c.Clear
    'Move through the data controls database. Get each record and
    'add t as an item to the passed combo box, c.
    While Not d.Recordset.EOF
        Last = d.Recordset.Fields("LastName")
        First = d.Recordset.Fields("FirstName")
        Phone = d.Recordset.Fields("Phone")
        'Concatenate the retrieved fields together into one
        'string and add it to the combo box
        c.AddItem Last & ", " & First & ", " & Phone
        'Move to the next record in the Data control
        d.Recordset.MoveNext
    Wend
    'Set the Combo to the first record
    c.ListIndex = 0
End Sub

'************************************************
'Sub/Function Name:GetGutter
'
'Arguments: GutterWidth As Integer
'
'
'Return:    A string the length of the the passed gutterwidth
'           filled with a padding character
'
'Remarks:   This function is used to provide a gutter string
'           for the appointment string entry.
'
'
'VB concept demonstrated: String()
'
'************************************************'
Public Function GetGutter(GutterWidth As Integer) As String
    Dim PadChar As String
    'Define the padding characer
    PadChar = " "
    'Pad the string with padding character, the length of gutterwidth
    GetGutter = String(GutterWidth, PadChar)
End Function

'************************************************
'Sub/Function Name: InitGlobalValues
'
'Arguments: None
'
'Return: None
'
'Remarks: This Sub is used to initialized the global variables
'         values.
'
'VB concept demonstrated:   Screen.TwipsPerPixelX (Y)
'                           TextWidth()
'                           Variables with global scope.
'
'************************************************'
Public Sub InitGlobalValues()
    Dim tWidth%
    
    tWidth% = frmMain.TextWidth("t")
    gGutterHorz = GUTTER_HORZ * Screen.TwipsPerPixelY
    gGutterVert = GUTTER_VERT * Screen.TwipsPerPixelX
    gColOneSize = 9
    gColTwoSize = 34
    gColThreeSize = 4
    
    gAppointmentField = (gColOneSize + gColTwoSize _
                            + gColThreeSize) * tWidth%
    gLeftFramesWidth = (gAppointmentField * (1 + _
                       (1 - LIST_RATIO_H)))
    ' Used to indicate an open appointment
    gOpenApptLine = String(gColTwoSize - 4, "-")
End Sub

'************************************************
'Sub/Function Name: CenterForm
'
'Arguments: f as Form       The form to be centered
'
'Return: None
'
'Remarks:   CenterForm centers a form, f to the screen.
'
'
'VB concept demonstrated: Passing an object as a sub parameter
'************************************************'
Public Sub CenterForm(f As Form)
    f.Left = (Screen.Width - f.Width) / 2
    f.Top = (Screen.Height - f.Height) / 2
End Sub

'************************************************
'Sub/Function Name: IsValidFile
'
'Arguments: f as String     A string representing a filename and
'                           filepath
'
'
'Return: True, if the file argument exists
'
'Remarks:   Use this function to determine if a file is present
'           on your system
'
'
'VB concept demonstrated: GetAttr()
'
'************************************************'
Public Function IsValidFile(f As String) As Boolean
    'Checks to make sure passed file is vbNormal
    On Error GoTo IsValidFileErr:
    If GetAttr(f) <> 0 Then IsValidFile = True
IsValidFileErr:
End Function

Sub Main()
    'Load the form and let it run the code in the
    ' Form_Load event handler
    Load frmMain
    'Intialize the contact list combo box
    Call InitComboAsDb(frmMain.cboName, frmMain.dataMain)
    'Fill the appointment list with today's appoinments
    Call GetDailyAppointments(CDbl(CDate(frmMain.FormDateString())), _
                              frmMain.lstSchedule, _
                              frmMain.dataMain, _
                              gAppointmentDelta)
    'Show the main form
    frmMain.Show
    'Set the mouse pointer back to an arrow
    frmMain.MousePointer = 0
End Sub



'************************************************
'Sub/Function Name: AttachName
'
'Arguments: sLastName   A string containing a last name
'                       value
'
'           sFirstName  A string containing a first name
'                       value
'
'           sPhone      A string containinga phone number
'                       value
'
'           d as Data   A Data control
'
'Return: None
'
'Remarks: AttachName attaches a retrieved contact record
'         to the appropriate text box controls.
'
'
'VB concept demonstrated:
'
'************************************************'
Public Sub AttachName(sLastName As String, _
                      sFirstName As String, _
                      sPhone As String, _
                      d As Data)
    Dim MySQL As String
    'Make the SQL statement retrieves a record based on the passed
    'strings sFirstName, sLastName and sPhone from the table, tblContacts.
    MySQL = "SELECT DISTINCTROW tblContacts.*, tblContacts.LastName, tblContacts.FirstName, tblContacts.Phone "
    MySQL = MySQL & "FROM tblContacts "
    MySQL = MySQL & "WHERE (((tblContacts.LastName)="
    MySQL = MySQL & DQ & sLastName & DQ & ") "
    MySQL = MySQL & "AND ((tblContacts.FirstName)=" & DQ & sFirstName & DQ & ") "
    MySQL = MySQL & "AND ((tblContacts.Phone)=" & DQ & sPhone & DQ & "))"
    'Set the SQL statement as a recordsource for the Data control, d
    d.RecordSource = MySQL
    'Connect the recordsource to the data control
    d.Refresh
    'Get the retrieved fields values and assign them to the
    'appropriate text fields
    frmMain.txtField(0).Text = d.Recordset.Fields("Company")
    frmMain.txtField(1).Text = d.Recordset.Fields("Address")
    frmMain.txtField(2).Text = d.Recordset.Fields("City")
    frmMain.txtField(3).Text = d.Recordset.Fields("State")
    frmMain.txtField(4).Text = d.Recordset.Fields("Zip")
    frmMain.txtField(5).Text = d.Recordset.Fields("tblContacts.Phone")
    frmMain.txtField(6).Text = d.Recordset.Fields("Fax")
    frmMain.txtField(7).Text = d.Recordset.Fields("Email")
End Sub



'************************************************
'Sub/Function Name: GetApptRecord
'
'Arguments: ap as Appt      A user defined type holding
'                           appointment data
'
'           d as Data       A data control
'
'Return: None
'
'Remarks:   GetApptRecord retrieves an appointment from the
'           appointments table as determined by the
'           appointment start time.
'
'
'VB concept demonstrated: Data.Recordsource
'                         Date.Refresh
'                         User defined types
'                         String concatentation
'                         Using SQL to retrieve records from a database
'************************************************
Public Sub GetApptRecord(ap As Appt, d As Data)
    Dim MySQL As String ' A string to hold the SQL string
    
    'Make an SQL statement that retrieves a record with
    'a specific start time as defined in the field, sTime of
    'the user defined type, ap
    MySQL = "SELECT tblAppointments.StartTime, "
    MySQL = MySQL & "tblAppointments.Description, "
    MySQL = MySQL & "tblAppointments.EndTime, *"
    MySQL = MySQL & " FROM tblAppointments "
    MySQL = MySQL & " WHERE (((tblAppointments.StartTime)="
    MySQL = MySQL & "#" & CStr(CDate(ap.sTime)) & "#))"
    'Assign the SQL statement to the data control's recordsource.
    d.RecordSource = MySQL
    'Hook the SQL statement up to the data control.
    d.Refresh
    'Set the retrieved end time to the end time field of the
    'user defined control
    ap.eTime = d.Recordset.Fields("EndTime")
    'Set the retrieved appointment description to the
    'comment field of the user defined control
    ap.Comment = d.Recordset.Fields("Description")
    End Sub
'************************************************
'Sub/Function Name: GetDailyAppointments
'
'Arguments: dblDayDate as Double    A double representing a day
'                                   in serial time notation.
'
'           lst as ListBox          A listbox control containing
'                                   appointments
'
'           d as Data               A Data control
'
'           Granularity as Integer  A integer representing the time
'                                   paritioning on the appointment
'                                   list.
'Return: None
'
'Remarks: GetDailyAppointments creates a list of appointments
'         for a given day.
'
'
'VB concept demonstrated:   ListBox.Clear
'                           ListBox.AddItem
'                           ListBox.List
'                           ListBox.ListIndex
'                           ListBox.TopIndex
'                           For...Next
'                           If...Then...Else
'                           Cstr()
'                           CLng()
'                           Instr()
'                           Left()
'                           Len()
'                           Format()
'                           Data.RecordSource
'                           Data.Refresh
'                           Data.Recordset.MoveFirst
'                           Data.Recordset.MoveLast
'                           Data.Recordset.RecordCount
'                           On Error Resume Next
'                           ReDim
'************************************************'
Public Sub GetDailyAppointments(dblDayDate As Double, _
                                     lst As ListBox, _
                                     d As Data, _
                                     Granularity As Integer)
    Dim MySQL As String
    Dim lDate As Long
    Dim lNextDate As Long
    Dim ap() As Appt
    Dim i%
    Dim j%
    Dim NumOfAppts%
    'If there is an error, keep going
    On Error Resume Next
    'Convert the Date to a long
    lDate = CLng(dblDayDate)
    'Add one to get the next day
    lNextDate = lDate + 1
    'Make the SQL statement that selects all the days greater
    'than today (from one nanosecond past midnite) and less than
    'then tommorrow.
    MySQL = "SELECT DISTINCTROW tblAppointments.StartTime, "
    MySQL = MySQL & "tblAppointments.EndTime , "
    MySQL = MySQL & "tblAppointments.Description "
    MySQL = MySQL & "FROM tblAppointments "
    MySQL = MySQL & "WHERE (((tblAppointments.StartTime) >="
    MySQL = MySQL & CStr(lDate) & ") And ((tblAppointments.EndTime)"
    MySQL = MySQL & " < " & CStr(lNextDate) & "))"
    MySQL = MySQL & " ORDER BY tblAppointments.StartTime"

    'Attach SQL to data control
    d.RecordSource = MySQL
    'Refresh to connect
    d.Refresh
    'Move through the recordset to get a count
    d.Recordset.MoveLast
    'Move back to the beginning of the database
    d.Recordset.MoveFirst
    
    'Make an Appt array to hold all the found appointments
    If d.Recordset.RecordCount > 0 Then
        NumOfAppts% = d.Recordset.RecordCount
        ReDim ap(NumOfAppts%)
    End If
    
    'Populate the Appointment array
    For i% = 0 To NumOfAppts% - 1
        ap(i%).sTime = d.Recordset.Fields("StartTime")
        ap(i%).eTime = d.Recordset.Fields("EndTime")
        ap(i%).Comment = d.Recordset.Fields("Description")
        d.Recordset.MoveNext
    Next i%
    'Make some time variables
    Dim h%
    Dim m%
    Dim hr As String
    Dim entry As String
    Dim g%
    'Set the granularity to a local variable
    g% = Granularity
    'Clear the list box
    lst.Clear
    'See InitAppointments to see how this string contatenation
    'algorithm works.
    For h% = 0 To 12
        For m% = 0 To 45 Step g%
            If Len(CStr(h%)) = 1 Then
                hr = "0" & CStr(h%)
            Else
                hr = CStr(h%)
            End If
            'Account for 12 PM
            If h% <> 12 Then
                If m% = 0 Then
                    entry = hr & ":" & "00 AM:"
                Else
                    entry = hr & ":" & CStr(m%) & " AM:"
                End If
            Else
                If m% = 0 Then
                    entry = hr & ":" & "00 PM:"
                Else
                    entry = hr & ":" & CStr(m%) & " PM:"
                End If
            End If
            lst.AddItem entry
        Next m%
    Next h%

     For h% = 1 To 11
            For m% = 0 To 45 Step g%
                If Len(CStr(h%)) = 1 Then
                    hr = "0" & CStr(h%)
                Else
                    hr = CStr(h%)
                End If
                    
                
                If m% = 0 Then
                    entry = hr & ":" & "00 PM:"
                Else
                    entry = hr & ":" & CStr(m%) & " PM:"
                End If
                lst.AddItem entry
            Next m%
        Next h%
    
    'Check the list items against the appointment array,
    'If there is a match, substitute the array item for the
    'line placeholder
    'Run through the list itesm
     For j% = 0 To lst.ListCount - 1
        'Take each item and run it through the Appointments array
        For i% = 0 To NumOfAppts - 1
            If Left(lst.List(j%), 8) = Format(ap(i%).sTime, "Medium Time") Then
                lst.List(j%) = Left(lst.List(j%), 11) & GetGutter(COLUMN_GUTTER) _
                    & ap(i%).Comment
            End If
        Next i%
        'Isolate the lines that do not have an Appt entry
        If Len(lst.List(j%)) <= Len(Left(lst.List(j%), 11 + COLUMN_GUTTER)) Then
            'Write a place holder line
            lst.List(j%) = Left(lst.List(j%), 11) & GetGutter(COLUMN_GUTTER) & gOpenApptLine
        End If
     Next j%
   
    'Set the top to 8 AM
    Dim strLook As String
    Dim strLookFor As String
    
    strLookFor = "08:00 AM:"
    For i% = 0 To lst.ListCount - 1
        strLook = lst.List(i%)
        If InStr(strLook, strLookFor) <> 0 Then
            lst.TopIndex = i%
        End If
    Next i%

    gAppointmentDelta = g%
    
End Sub

'************************************************
'Sub/Function Name: WriteAppointment
'
'Arguments: a As Appt   As user defined type containing appointment
'                       data.
'
'           d As Data   A data control attached to a database contain-
'                       ing the appointments table.
'
'Return: True, if appointment write is successful
'
'Remarks:   WriteAppointment encapsulates the writing of a
'           record to a database.
'
'
'VB concept demonstrated:   Data.Recordsource
'                           Data.Refresh
'                           Data.Recordset.AddNew
'                           DataRecordset.Fields()
'                           DataRecordSet.Update
'                           On Error
'************************************************'
Public Function WriteAppointment(a As Appt, d As Data) As Boolean
    Dim MySQL As String
    Dim i%
    Dim CurAppt() As Appt
    Dim lDayDate As Long
'If the write causes an error, send the flow out of the function.
On Error GoTo WriteAppointmentErr:
        
    'If the date is OK, then
    If CheckValidApptWrite(a, d) Then
        'Reconnnect the data recordsource to the appointment table
        d.RecordSource = "tblAppointments"
        'Refresh the database
        d.Refresh
        'Add a new record
        d.Recordset.AddNew
        'Write the appointment time
        d.Recordset.Fields("StartTime") = a.sTime
        'Write the appointment end time
        d.Recordset.Fields("EndTime") = a.eTime
        'Write the appointment description
        d.Recordset.Fields("Description") = a.Comment
        'Do the write
        d.Recordset.Update
        'Report that things are good
        WriteAppointment = True
    End If
'If the write causes an error, the program flow will drop to here.
'The function will NOT be set to true, we'll exit the function and
'eventually an error will flagged.
WriteAppointmentErr:
End Function
'************************************************
'Sub/Function Name: InitAppointments
'
'Arguments: Granualarity as Integer     An integer representing
'                                       the time partition of the
'                                       appointment list.
'
'           lst as ListBox              A listbox containing the
'                                       daily appointments.
'Return:
'
'Remarks:   InitAppointments is used to create the list of appointments
'           that get added to the list box, lst. The sub creates
'           strings that begin with a time stamp of the form,
'           hh:mm: AM/PM. The sub will query the appointments table
'           for all existing appointments for that day. If it finds
'           an appoinment it will be added to the appointment list,
'           at the appropriate time.
'
'           After all appointments have been entered and all times
'           listed for a 24 day, the sub, sets the top of the list
'           to be 08:00 AM.
'
'VB concept demonstrated:   ListBox.Clear
'                           ListBox.AddItem
'                           ListBox.List
'                           ListBox.ListIndex
'                           ListBox.TopIndex
'                           For...Next
'                           If...Then...Else
'                           Cstr()
'                           Instr()
'                           Len()
'************************************************'
Public Sub InitAppointments(Granularity As Integer, _
                        lst As ListBox)
    Dim h%
    Dim m%
    Dim hr As String
    Dim entry As String
    Dim g%
    'Set the granularity to a local variable
    g% = Granularity
    'Clear the list box
    lst.Clear
    'Go through the morning hours
    For h% = 0 To 12
        'Go through the minutes by the steps as defined
        'the granualarity variable, g.
        For m% = 0 To 45 Step g%
            'If the length of the hour is less then 2 digits (i.e., 9)
            'then add a zero to the left of the single digit
            If Len(CStr(h%)) = 1 Then
                'Oh, yes make it a string
                hr = "0" & CStr(h%)
            Else
                'even if it is two digits
                hr = CStr(h%)
            End If
            'Add the "AM" part to the string
            If h% <> 12 Then
                If m% = 0 Then
                    'Add double zeros to the hour, in addition
                    'to the "AM"
                    entry = hr & ":" & "00 AM:"
                Else
                    'If it is not right on the hour, just add
                    'the "AM" to the end of the string
                    entry = hr & ":" & CStr(m%) & " AM:"
                End If
            Else
                'If it is the twelfth hour it is really a "PM"
                If m% = 0 Then
                    'Right on the hour, add a double zero and the "PM"
                    entry = hr & ":" & "00 PM:"
                Else
                    'Off the hour, just "PM"
                    entry = hr & ":" & CStr(m%) & " PM:"
                End If
            End If
            'Make the entry string
            entry = entry & GetGutter(COLUMN_GUTTER) & gOpenApptLine
            'Add the string to list
            lst.AddItem entry
        Next m%
    Next h%
    'Now the do the evening hours the same as you did the morning
    For h% = 1 To 11
            For m% = 0 To 45 Step g%
                If Len(CStr(h%)) = 1 Then
                    hr = "0" & CStr(h%)
                Else
                    hr = CStr(h%)
                End If
                                    
                If m% = 0 Then
                    entry = hr & ":" & "00 PM:"
                Else
                    entry = hr & ":" & CStr(m%) & " PM:"
                End If
                entry = entry & GetGutter(COLUMN_GUTTER) & gOpenApptLine
                lst.AddItem entry
            Next m%
        Next h%
    'Set the top to 8 AM
    Dim strLook As String
    Dim strLookFor As String
    Dim i%
    strLookFor = "08:00 AM:"
    
    'Take a look at every item in the list
    For i% = 0 To lst.ListCount - 1
        strLook = lst.List(i%)
        'If it matches the time to look for, set that line
        'as the top line.
        If InStr(strLook, strLookFor) <> 0 Then
            lst.TopIndex = i%
        End If
    Next i%
    'Set the time increment semaphore. You'll need it to
    'figure out how to make end times.
    gAppointmentDelta = g%
End Sub
'************************************************
'Sub/Function Name: SetAppointmentString
'
'Arguments: lblStart as Label   A label containing the start time of
'                               an appointment
'
'           lblEnd As Label     A label containng the end time of an
'                               an appointment
'
'           strDate As String   A string containing the full date of
'                               an appointment
'
'           d As Data           A data control containing the
'                               appointment's table
'
'           lst As ListBox      A list box containing a list of
'                               appointments.
'
'           txt As TextBox      A text box to which the appointment
'                               comment will be written
'
'Return: An integer indicating the state of the write attempt
'           APPOINTMENT_MADE, successful write
'           ERR_MATCHING_TIME, start time matches end tim
'           ERR_NULL_APPOINTMENT, no comment string
'           ERR_APPOINTMENT_CONFLICT, appointment already exists
'
'Remarks:   SetAppointmentString is used take data entered in
'           frame, frEnterAppt, write it to the database and enter
'           the appointment in the appointment list, lstSchedule.
'           If an error is encountered the function will return an
'           error value.
'
'
'VB concept demonstrated:   CDbl()
'                           CDate()
'                           ListIndex property
'                           List property
'************************************************'
Public Function SetAppointmentString(lblStart As Label, _
                                     lblEnd As Label, _
                                     strDate As String, _
                                     d As Data, _
                                     lst As ListBox, _
                                     txt As TextBox) As Integer
    Dim a As Appt
    Dim Appt As String
    Dim dblStartTime As Double
    Dim dblEndTime As Double
    
    'Make DBL start date
    dblStartTime = CDbl(CDate(strDate) + CDate(Trim(lblStart.Caption)))
    'Make a DBL end date
    dblEndTime = CDbl(CDate(strDate) + CDate(Trim(lblEnd.Caption)))
        
    'Init up the User Defined appointment array
    a.sTime = dblStartTime
    a.eTime = dblEndTime
    a.Comment = txt.Text
    'Make the string that will be written to the appointment list
    Appt = lblStart.Caption & ": " & txt.Text
    If gbEditOn Then
        'The end time matching the start time is a no, no
        If lblStart.Caption = lblEnd.Caption Then
            SetAppointmentString = ERR_MATCHING_TIME
            Exit Function
        'A blank comment is bad!!!
        ElseIf txt.Text = "" Then
            SetAppointmentString = ERR_NULL_APPOINTMENT
            Exit Function
        Else
            'OK, you have an appointment that you can attempt to
            'make. Try to write it.
            If WriteAppointment(a, d) Then
                'If the write as successful, you will get a TRUE
                'return from WriteAppointment()
                SetAppointmentString = APPOINTMENT_MADE
                'The write was done, add it to the appointment
                'list box.
                lst.List(gCurrentApptIndex) = Appt
                'Turn off the gbEditOn semaphore
                gbEditOn = False
            Else
                'A bad write will return FALSE, report it.
                SetAppointmentString = ERR_APPOINTMENT_CONFLICT
            End If
        End If
    End If
End Function

'************************************************
'Sub/Function Name: CheckValidApptWrite
'
'Arguments: a as Appt   A user defined structure, defined in the
'                       Declarations section of this module
'
'           d as Data   A Data control
'
'
'Return: True, if a data write is possible
'
'Remarks:   CheckValidApptWrite checks to make sure that
'           as appointment as passed in user defined type,
'           Appt does not conflict with an existing
'           appointment.
'
'
'VB concept demonstrated:   Using an SQL statement as a
'                               Data control recordsource
'                           CLng()
'                           CStr()
'                           Instr()
'                           Data.Recordsource
'                           Data.Refresh
'                           Data.Recordset.MoveFirst
'                           Data.Recordset.MoveLast
'                           Date.Recordset.RecordCount
'                           ReDim
'************************************************'
Public Function CheckValidApptWrite(a As Appt, d As Data) As Boolean
    Dim lDayDate As Double
    Dim MySQL As String
    Dim NumOfRecs%
    Dim i%
    Dim DecPos%
    'Find the day date of the Appointment
    
    'Get rid of the trailing decimal.
    '(Note: If it is greater than .5, it really messes things up
    ' because it will round into another day.)
    
    'Get the position of the decimal point
    DecPos% = InStr(CStr(a.sTime), ".")
    'Remove the whole number from the decimal places of
    'a.sTime and convert the whole number string into a data
    'type, long.
    lDayDate = CLng(Left(CStr(a.sTime), DecPos% - 1))
    'Make an SQL statement to look at all the records for that day
    'This is sort of shortcut so that we don't need to run the whole
    'show
    MySQL = "SELECT DISTINCTROW tblAppointments.StartTime, "
    MySQL = MySQL & "tblAppointments.EndTime From tblAppointments "
    MySQL = MySQL & "WHERE (((tblAppointments.StartTime)>=  #"
    MySQL = MySQL & CStr(CDate(lDayDate)) & "#)"
    MySQL = MySQL & " AND ((tblAppointments.StartTime)< #"
    MySQL = MySQL & CStr(CDate(lDayDate + 1)) & "#))"
    'Assign the SQL statement as the recordsource for the data
    'control
    d.RecordSource = MySQL
    'Hook up the SQL statement to the data control
    d.Refresh
    'If you return no records this means that the day has all open times
    NumOfRecs% = d.Recordset.RecordCount
    'If no records are retrieved you know that you have a unique
    'appointment time.
    If (NumOfRecs%) = 0 Then
        'Report the date as good
        CheckValidApptWrite = True
        'Leave the function
        Exit Function
    End If
    
    'If you do retrieve some records, you now have to make sure
    'that the date you are querying for does not occur at the
    'starting time or within an appointment that already exists.

    'Make an array of all appointments within submitted day
    ReDim ClosedAppts(NumOfRecs%)
    'Fill the array
    'If dStart >= xStart AND dStart < xEnd then
    For i% = 0 To NumOfRecs% - 1
        If a.sTime >= d.Recordset.Fields("StartTime") And _
                a.sTime < d.Recordset.Fields("EndTime") Then
            'I am hosed
            CheckValidApptWrite = False
            Exit Function
        End If
        'If dEnd > xStart AND dEnd <= xEnd then
        If a.eTime > d.Recordset.Fields("StartTime") And _
                a.eTime <= d.Recordset.Fields("EndTime") Then
            'I am hosed
            CheckValidApptWrite = False
            Exit Function
        End If
        'move to the next record
        d.Recordset.MoveNext
    Next i%
    
    'If you get to here, there is no conflict
    CheckValidApptWrite = True
End Function

'************************************************
'Sub/Function Name: DeleteApptRecord
'
'Arguments: dblStartTime as Double  A long representing an appointment
'                                   Start Time in serial time notation.
'
'           d as Data               A Data control
'
'Return: None
'
'Remarks: DeleteRecord encapsulates the Delete
'         method of a recordset.
'
'VB concept demonstrated:   Recordset.Edit
'                           Recordset.Delete
'                           MsgBox()
'                           User defined types
'************************************************'
Public Sub DeleteApptRecord(dblStartTime As Double, d As Data)
    Dim ap As Appt
    Dim Msg As String
    'Set the start time of the ap element
    ap.sTime = dblStartTime
    'Retrieve the appointment record
    Call GetApptRecord(ap, d)
    'Make a prompt to confirm the delete
    Msg = "Are you sure that you want to Delete the appointment: "
    Msg = Msg & vbCrLf & vbCrLf
    Msg = Msg & ap.Comment
    'Confirm the delete
    If MsgBox(Msg, vbQuestion + vbYesNo, "Delete Appointment?") = vbYes Then
        'If yes, delete the record
        d.Recordset.Edit
        d.Recordset.Delete
    End If
End Sub
